#' @keywords internal
#' @aliases arcopt-package NULL
"_PACKAGE"

## usethis namespace: start
#' @useDynLib arcopt, .registration = TRUE
#' @importFrom Rcpp evalCpp
#' @importFrom Rcpp sourceCpp
## usethis namespace: end
NULL
