% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arcopt-package.R
\docType{package}
\name{arcopt-package}
\alias{arcopt-package}
\title{arcopt: Adaptive Regularization using Cubics for Optimization}
\description{
Implements cubic regularization methods (ARC) for local optimization problems common in statistics and applied research. Provides robust handling of ill-conditioned, nonconvex, and indefinite Hessian problems with automatic saddle point escape. Supports box constraints; linear equality constraints are planned for a future release.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/marcus-waldman/arcopt}
  \item Report bugs at \url{https://github.com/marcus-waldman/arcopt/issues}
}

}
\author{
\strong{Maintainer}: Marcus Waldman \email{marcus.waldman@cuanschutz.edu}

}
\keyword{internal}
