% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/flat_ridge.R
\name{init_flat_ridge_state}
\alias{init_flat_ridge_state}
\title{Initialize Flat-Ridge Detector State}
\usage{
init_flat_ridge_state(window = 10)
}
\arguments{
\item{window}{Integer sliding-window length (default 10)}
}
\value{
A list with empty diagnostic vectors and the window size.
}
\description{
Initialize Flat-Ridge Detector State
}
\seealso{
\code{\link{update_flat_ridge_state}},
\code{\link{check_flat_ridge_trigger}}
}
\keyword{internal}
